// Squawk messenger. 
// Copyright (C) 2019  Yury Gubich <blue@macaw.me>
// 
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

#ifndef CORE_OMEMOHANDLER_H
#define CORE_OMEMOHANDLER_H

#include <QXmppOmemoStorage.h>
#include <cache.h>

Q_DECLARE_METATYPE(QXmppOmemoStorage::OwnDevice);
Q_DECLARE_METATYPE(QXmppOmemoStorage::Device);
Q_DECLARE_METATYPE(QXmppOmemoStorage::SignedPreKeyPair);

namespace Core {
class Account;

class OmemoHandler : public QXmppOmemoStorage
{
public:
    OmemoHandler(Account* account);
    ~OmemoHandler() override;

    QFuture<OmemoData> allData() override;

    QFuture<void> setOwnDevice(const std::optional<OwnDevice> &device) override;

    QFuture<void> addSignedPreKeyPair(uint32_t keyId, const SignedPreKeyPair &keyPair) override;
    QFuture<void> removeSignedPreKeyPair(uint32_t keyId) override;

    QFuture<void> addPreKeyPairs(const QHash<uint32_t, QByteArray> &keyPairs) override;
    QFuture<void> removePreKeyPair(uint32_t keyId) override;

    QFuture<void> addDevice(const QString &jid, uint32_t deviceId, const Device &device) override;
    QFuture<void> removeDevice(const QString &jid, uint32_t deviceId) override;
    QFuture<void> removeDevices(const QString &jid) override;

    QFuture<void> resetAll() override;

private:
    static QFuture<void> emptyVoidFuture();

private:
    Account* acc;
    std::optional<OwnDevice> ownDevice;
    DataBase db;
    DataBase::Cache<QString, QVariant>* meta;
    DataBase::Cache<QString, QHash<uint32_t, Device>>* devices;
    DataBase::Cache<uint32_t, QByteArray>* preKeyPairs;
    DataBase::Cache<uint32_t, QXmppOmemoStorage::SignedPreKeyPair>* signedPreKeyPairs;
};

}

QDataStream& operator << (QDataStream &out, const QXmppOmemoStorage::Device& device);
QDataStream& operator >> (QDataStream &in, QXmppOmemoStorage::Device& device);

QDataStream& operator << (QDataStream &out, const QXmppOmemoStorage::OwnDevice& device);
QDataStream& operator >> (QDataStream &in, QXmppOmemoStorage::OwnDevice& device);

QDataStream& operator << (QDataStream &out, const QXmppOmemoStorage::SignedPreKeyPair& device);
QDataStream& operator >> (QDataStream &in, QXmppOmemoStorage::SignedPreKeyPair& device);

#endif // CORE_OMEMOHANDLER_H
